-- Copyright (c) 2024 - Ognjen 'xolatile' Milan Robovic
--
-- GNU General Public Licence (version 3 or later)

with core, ui, resource, item, unit, construction;

package body world is

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

	view_reach : constant integer := 48;

	dark : core.sprite;

                                             ------------------------------------------------------------------------------------------

	procedure configure is
	begin
		core.echo (core.comment, "Configuring world components...");
		--
		tiles := core.import_sprite ("./sprite/world/terrain/terrain.png", 1, 1);
		dark  := core.import_sprite ("./sprite/dark.png",                  1, 1);
		--
		for index in landmark_index loop
			declare file : constant string := core.lowercase (index'image);
			begin
				landmarks (index) := core.import_sprite ("./sprite/world/landmark/" & file & ".png", trait (index).frames, 1);
			end;
		end loop;
	end configure;

                                             ------------------------------------------------------------------------------------------

	procedure make (index : in biome; width, height : in natural) is
	begin
		core.echo (core.comment, "-- Procedurally generating new map...");
		--
		core.echo (core.comment, "-- -- Map type       : " & index'image);
		core.echo (core.comment, "-- -- Map width      :"  & width'image);
		core.echo (core.comment, "-- -- Map height     :"  & height'image);
		core.echo (core.comment, "-- -- Landmark count :"  & landmark_limit'image);
		--
		map.kind   := index;
		map.width  := width;
		map.height := height;
		--
		map.tiles         := new tile_array   (0 .. map.width - 1, 0 .. map.height - 1);
		map.clips         := new clip_array   (0 .. map.width - 1, 0 .. map.height - 1);
		map.views         := new view_array   (0 .. map.width - 1, 0 .. map.height - 1);
		map.landmarks     := new entity_array (1 .. landmark_limit);
		map.constructions := new entity_array (1 .. 30);
		map.items         := new entity_array (1 .. 60);
		map.units         := new entity_array (1 .. 90);
		--
		for x in 0 .. width - 1 loop
			for y in 0 .. height - 1 loop
				map.tiles (x, y) := (core.random (0, 23) * core.random (0, 23)) mod 24;
				map.clips (x, y) := false;
				map.views (x, y) := false;
			end loop;
		end loop;
		--
		for index in 1 .. landmark_limit loop
			map.landmarks (index).index := core.random (0, 9);
			map.landmarks (index).x     := core.random (6, map.width  - 6);
			map.landmarks (index).y     := core.random (6, map.height - 6);
			--
			if trait (landmark_index'val (map.landmarks (index).index)).clip then
				declare reach_x : constant natural := landmarks (landmark_index'val (map.landmarks (index).index)).width  / core.base;
				        reach_y : constant natural := landmarks (landmark_index'val (map.landmarks (index).index)).height / core.base;
				begin
					for x in 0 .. reach_x - 1 loop
						for y in 0 .. reach_y - 1 loop
							map.clips (map.landmarks (index).x + x, map.landmarks (index).y + y) := true;
						end loop;
					end loop;
				end;
			end if;
		end loop;
		--
		for index in 1 .. 30 loop
			map.constructions (index).index := core.random (0, construction.count - 1);
			map.constructions (index).x     := core.random (6, map.width          - 6);
			map.constructions (index).y     := core.random (6, map.height         - 6);
			--
			declare reach_x : constant natural := construction.sprite (construction.enumeration'val (map.constructions (index).index)).width  / core.base;
			        reach_y : constant natural := construction.sprite (construction.enumeration'val (map.constructions (index).index)).height / core.base;
			begin
				for x in 0 .. reach_x - 1 loop
					for y in 0 .. reach_y - 1 loop
						map.clips (map.constructions (index).x + x, map.constructions (index).y + y) := true;
					end loop;
				end loop;
			end;
		end loop;
		--
		for index in 1 .. 60 loop
			map.items (index).index := core.random (0, item.count - 1);
			map.items (index).x     := core.random (0, map.width  - 1);
			map.items (index).y     := core.random (0, map.height - 1);
		end loop;
		--
		for index in 1 .. 90 loop
			map.units (index).index := core.random (0, unit.count - 1);
			map.units (index).x     := core.random (0, map.width  - 1);
			map.units (index).y     := core.random (0, map.height - 1);
			--
			map.clips (map.units (index).x, map.units (index).y) := true;
		end loop;
		--
		core.echo (core.success, "Finished procedurally generating new map.");
	end make;

                                             ------------------------------------------------------------------------------------------

	procedure draw is
		u : integer := 0;
		v : integer := 0;
		--
		offset : core.vector := ((core.window_width  - core.base) / 2,
		                         (core.window_height - core.base) / 2);
	begin
		view;
		--
		ui.draw_icon_menu (x      => core.base * core.zoom * (-1- core.camera.x) + offset.x,
		                   y      => core.base * core.zoom * (-1- core.camera.y) + offset.y,
		                   width  => core.base * core.zoom * (map.width  + 2),
		                   height => core.base * core.zoom * (map.height + 2));
		--
		for vertical in 0 .. map.height - 1 loop
			exit when offset.y + (vertical - core.camera.y) * core.base * core.zoom > core.window_height;
			--
			for horizontal in 0 .. map.width - 1 loop
				exit when offset.x + (horizontal - core.camera.x) * core.base * core.zoom > core.window_width;
				--
				if map.views (horizontal, vertical) then
					u := core.base * biome'pos (map.kind) * 4;
					v := core.base * map.tiles (horizontal, vertical);
					--
					core.draw (data   => tiles,
					           x      => offset.x + (horizontal - core.camera.x) * core.base * core.zoom,
					           y      => offset.y + (vertical   - core.camera.y) * core.base * core.zoom,
					           u      => u,
					           v      => v,
					           width  => core.base,
					           height => core.base);
					if  core.cursor.x > offset.x + (horizontal - core.camera.x    ) * core.base * core.zoom - 6
					and core.cursor.x < offset.x + (horizontal - core.camera.x + 1) * core.base * core.zoom + 6
					and core.cursor.y > offset.y + (vertical   - core.camera.y    ) * core.base * core.zoom - 6
					and core.cursor.y < offset.y + (vertical   - core.camera.y + 1) * core.base * core.zoom + 6
					and core.cursor_mode = 1 and not ui.prioritize then
						core.camera.x := horizontal;
						core.camera.y := vertical;
						core.cursor_mode := 0;
					end if;
				end if;
			end loop;
		end loop;
		--
		for index in 1 .. landmark_limit loop
			if map.views (map.landmarks (index).x, map.landmarks (index).y) then
				core.draw (data => landmarks (landmark_index'val (map.landmarks (index).index)),
				           x    => offset.x + (map.landmarks (index).x - core.camera.x) * core.base * core.zoom,
				           y    => offset.y + (map.landmarks (index).y - core.camera.y) * core.base * core.zoom);
			end if;
		end loop;
		--
		for index in 1 .. 30 loop
			if map.views (map.constructions (index).x, map.constructions (index).y) then
				construction.draw (construction.enumeration'val (map.constructions (index).index),
				                   offset.x + (map.constructions (index).x - core.camera.x) * core.base * core.zoom,
				                   offset.y + (map.constructions (index).y - core.camera.y) * core.base * core.zoom);
			end if;
		end loop;
		--
		for index in 1 .. 60 loop
			if map.views (map.items (index).x, map.items (index).y) then
				item.draw (item.enumeration'val (map.items (index).index),
				           core.idle,
				           offset.x + (map.items (index).x - core.camera.x) * core.base * core.zoom,
				           offset.y + (map.items (index).y - core.camera.y) * core.base * core.zoom);
			end if;
		end loop;
		--
		for index in 1 .. 90 loop
			if map.views (map.units (index).x, map.units (index).y) then
				unit.full_draw (unit.list (map.units (index).index),
				                offset.x + (map.units (index).x - core.camera.x) * core.base * core.zoom,
				                offset.y + (map.units (index).y - core.camera.y) * core.base * core.zoom);
			end if;
		end loop;
		--
		for vertical in 0 .. map.height - 1 loop
			exit when offset.y + (vertical - core.camera.y) * core.base * core.zoom > core.window_height;
			--
			for horizontal in 0 .. map.width - 1 loop
				exit when offset.x + (horizontal - core.camera.x) * core.base * core.zoom > core.window_width;
				--
				if (horizontal - core.camera.x) ** 2 + (vertical - core.camera.y) ** 2 > view_reach then
					core.draw (data   => dark,
					           x      => offset.x + (horizontal - core.camera.x) * core.base * core.zoom,
					           y      => offset.y + (vertical   - core.camera.y) * core.base * core.zoom);
				end if;
			end loop;
		end loop;
	end draw;

                                             ------------------------------------------------------------------------------------------

	procedure view is
		offset : core.vector := ((core.window_width  - core.base) / 2,
		                         (core.window_height - core.base) / 2);
	begin
		for vertical in 0 .. map.height - 1 loop
			exit when offset.y + (vertical - core.camera.y) * core.base * core.zoom > core.window_height;
			--
			for horizontal in 0 .. map.width - 1 loop
				exit when offset.x + (horizontal - core.camera.x) * core.base * core.zoom > core.window_width;
				--
				if not ((horizontal - core.camera.x) ** 2 + (vertical   - core.camera.y) ** 2 > view_reach * 2) then
					map.views (horizontal, vertical) := true;
				end if;
			end loop;
		end loop;
	end view;

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

end world;
